@@ -5,6 +5,7 @@ from bossman.errors import BossmanError
 from bossman.logging import get_class_logger
 from datetime import datetime
 import gitdb.exc
+import yaml
 
 def true(*args, **kwargs):
   return True
@@ -54,6 +55,14 @@ class Change(ABC):
   def path(self):
     return self.diff.b_path
 
+  @property
+  def basename(self):
+    return basename(self.path)
+
+  @property
+  def change_type(self):
+    return self.diff.change_type
+
   @property
   def contents(self):
     return self.diff.b_blob.data_stream.read()
@@ -70,7 +79,7 @@ class Change(ABC):
     yield "[{color}]{change_type} {name}[{color}]".format(
       color=color,
       change_type=self.diff.change_type,
-      name=basename(self.path),
+      name=self.basename,
     )
 
 class AddChange(Change):
@@ -93,7 +102,32 @@ class DeleteChange(Change):
   def contents(self):
     return self.diff.a_blob.data_stream.read()
 
+class Notes:
+  def __init__(self, commit: git.Commit, ns: str = None):
+    self.commit = commit
+    self.ns = ns.strip("/")
 
+  def set(self, **kwargs):
+    args = []
+    if self.ns is not None:
+      args.extend(("--ref", self.ns))
+    args.extend(("append", "-m", yaml.safe_dump(kwargs), self.commit.hexsha))
+    self.commit.repo.git.notes(*args)
+
+  def get(self, k: str, default=None):
+    raw = self.read()
+    notes = yaml.safe_load(raw if raw else '{}')
+    return notes.get(k, default)
+
+  def read(self):
+    args = []
+    if self.ns is not None:
+      args.extend(("--ref", self.ns))
+    args.extend(("show", self.commit.hexsha))
+    try:
+      return self.commit.repo.git.notes(*args)
+    except git.GitCommandError:
+      return None
 
 class Revision:
   def __init__(self, commit: git.Commit, diffs: git.Diff):
@@ -141,6 +175,7 @@ class Revision:
   def author_email(self) -> str:
     return self.commit.author.email
 
+  @property
   def affected_paths(self) -> list:
     return self.changes.keys()
 
@@ -163,6 +198,9 @@ class Revision:
   def get_change(self, path: str) -> Change:
     return self.changes.get(path, None)
 
+  def get_notes(self, ns: str = None) -> Notes:
+    return Notes(self.commit, ns)
+
   def __str__(self):
     s = "[{}] {} {} | {}".format(self.id, self.short_message, self.author_name, self.date)
     if len(self.changes):
@@ -172,7 +210,7 @@ class Revision:
     return s
 
   def __rich_console__(self, console, options):
-    yield "[b][{}][/b] {} | {} {}".format(self.id, self.short_message, self.author_name, self.date)
+    yield "[b]\[{}][/b] {} | {} {}".format(self.id, self.short_message, self.author_name, self.date)
     # for change in self.changes.values():
     #   yield change
 
@@ -197,13 +235,25 @@ class Repo:
     self.logger = get_class_logger(self)
     self._repo = git.Repo(root)
 
+  def config_writer(self):
+    return self._repo.config_writer()
+
+  def rev_parse(self, rev: str) -> str:
+    # We can't use Repo.rev_parse since it doesn't support
+    # short hexshas or indirect refs such as tag names or branches.
+    try:
+      hexsha = self._repo.git.rev_parse(rev)
+      return self._repo.rev_parse(hexsha)
+    except git.GitCommandError:
+      raise BossmanError("failed to resolve revision {}, please make sure it is a valid commit/tag name".format(rev))
+
   def get_paths(self, rev: str = "HEAD", predicate = true) -> list:
     """
     Lists the paths versioned for revision {rev}, optionally filtered
     by {predicate}.
     """
     try:
-      commit = self._repo.rev_parse(rev)
+      commit = self.rev_parse(rev)
     except gitdb.exc.BadName:
       raise BossmanError("This branch likely has no commits yet.")
 
@@ -212,8 +262,26 @@ class Repo:
     visitor(commit.tree)
     return paths
 
-  def get_last_revision(self, paths: list, rev: str = "HEAD") -> Revision:
+  def get_head(self):
+    """
+    Returns the HEAD revision, or None if this is the first commit.
+    """
     try:
+      commit = next(self._repo.iter_commits())
+      prev = git.NULL_TREE
+      if commit.parents:
+        prev = commit.parents[0]
+      diffs = commit.diff(prev, R=True)
+      return Revision(commit, diffs)
+    except StopIteration:
+      return None
+
+  def get_last_revision(self, paths: list, rev: str = "HEAD") -> Revision:
+    """
+    Returns the last revision in {rev}'s ancestry to have affected {paths}.
+    """
+    try:
+      rev = self.rev_parse(rev)
       commit = next(self._repo.iter_commits(rev, paths=paths))
       prev = git.NULL_TREE
       if commit.parents:
@@ -224,7 +292,10 @@ class Repo:
       return None
 
   def get_revision(self, rev: str, paths: list = None):
-    commit = self._repo.rev_parse(rev)
+    """
+    Returns the revision {rev}, with diffs for {paths}.
+    """
+    commit = self.rev_parse(rev)
     prev = git.NULL_TREE
     diffs = []
     if commit.parents:
@@ -233,6 +304,10 @@ class Repo:
     return Revision(commit, diffs)
 
   def get_revisions(self, since_rev: str = None, until_rev: str = "HEAD",  paths: list = None) -> list:
+    """
+    Returns all revisions having affected {paths} in the ancestry of {until_rev}, bounded by {since_rev} if
+    specified.
+    """
     commitRange = ("{since_rev}..{until_rev}".format(since_rev=since_rev, until_rev=until_rev)
       if since_rev
       else until_rev)
